
// Simple Average
// by James Lewis
// Use this code to compare to MMA example found here:
// www.baldengineer.com/mma
//
// Connect a wire between pin 5 (pwmPin) and analog 0 (analogPin)
// Open the serial monitor. Send the + and - characters to change
// the speed. (you can type in multiples like '+++' to change quickly)
//
// The "average" voltage valued display is the PWM average

// Pins
const byte pwmPin = 5;
const byte analogPin = A0;

// Let's compare an average of 100
const int averageCount = 100;

// So we can change the PWM value
int PWMvalue = 128; //~50% Duty

// Variables for the Modified Moving Average
unsigned int simpleAverageHistory[averageCount];
float simpleAverage;
int simpleAveragePointer = 0;

// Timer variables
unsigned long previousMillis = 0;
unsigned int  printInterval = 500;
unsigned long startTime = 0;
unsigned long stopTime = 0;

void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  pinMode(analogPin, INPUT); // not necessary, for FYI
  pinMode(pwmPin, OUTPUT); // not neceessary, for FYI
  pinMode(13, OUTPUT);
  digitalWrite(13, LOW);
  
  // Start the PWM waveform
  analogWrite(pwmPin, PWMvalue); // ~50% Duty Cycle

  // Pre-load Simple Average
  Serial.print(F("Loading initial array..."));
  for (int x=0; x < averageCount; x++)
    simpleAverageHistory[x] = analogRead(analogPin);
  Serial.println(F("done!"));
    
  // Calculate inital average by summing the array
  // and then dividing by number of elements
  // this could be skipped, i like to do it for fun
  float temporaryCount = 0;
  Serial.print(F("Calculating initial average..."));
  for (int y=0; y < averageCount; y++)
    temporaryCount = temporaryCount + (float)simpleAverageHistory[y];
  simpleAverage = temporaryCount / (float)averageCount;
  Serial.println(F("done!"));
}


void loop() {
  // check for serial control
  handleSerial();
  
  // each interation of loop, update moving average
  // Get a new sample
  unsigned int currentValue = analogRead(analogPin);

  // store the currentValue in a rotating buffer
  // around the array, so we have a "moving" history
  // move the pointer
  simpleAveragePointer++;
  if (simpleAveragePointer > averageCount)
    simpleAveragePointer = -1;
  simpleAverageHistory[simpleAveragePointer] = currentValue;
  
  startTime = micros();
  
  // Calculate the average by summing the array
  // and then dividing by number of elements
  // this process is repeated every time
  float temporaryCount = 0;
  for (int x=0; x < averageCount; x++)
    temporaryCount = temporaryCount + (float)simpleAverageHistory[x];
  
  simpleAverage = temporaryCount / (float)averageCount;

  stopTime = micros();

  if (millis() - previousMillis >= printInterval) {
    
    Serial.print(F("currentValue: "));
    Serial.println(currentValue);
    Serial.print(F("PWM Value: "));
    Serial.println(PWMvalue);
    Serial.print(F("Simple Average: "));
    Serial.println(simpleAverage);
    Serial.print("Calculation time: ");
    Serial.print(stopTime - startTime);
    Serial.println(" us");
    Serial.println();
    Serial.flush();

    // reset the millis clock
    previousMillis = millis();
    
  }

}

void handleSerial() {
  // we only care about two characters to change the pwm
  if (Serial.available() > 0) {
    switch (Serial.read()) {
      case '+':
        PWMvalue = PWMvalue + 32;
        if (PWMvalue >= 255)
          PWMvalue = 255;
      break;
      case '-':
        PWMvalue = PWMvalue - 32;
        if (PWMvalue <= 0)
          PWMvalue = 0;
      break;
    }
    analogWrite(pwmPin, PWMvalue);
  }
}

